#!/usr/bin/env python3
"""Create a notebook containing code from a script.
Run as:  python to_noteebook.py my_script.py
"""
import argparse
import os

import nbformat.v4

ACCEPTED_CHARS = ['#-', '# -']
MARKDOWN_CHARS = ['#|', '# |']
ACCEPTED_CHARS.extend(MARKDOWN_CHARS)

def new_cell(nb, cell, markdown=False):
    """ Create a new cell
    Parameters
    ----------
    nb: nbformat.notebooknode.NotebookNode
        Notebook to write to, as produced by nbformat.v4.new_notebook()
    cell: str
        String to write to the cell
    markdown: boolean, optional, (default False)
        Whether to create a markdown cell, or a code cell
    """
    cell = cell.rstrip().lstrip()
    if cell:
        if markdown:
            cell = nbformat.v4.new_markdown_cell(cell)
        else:
            cell = nbformat.v4.new_code_cell(cell)
        nb.cells.append(cell)
    return ''

def str_starts_with(string, options):
    for opt in options:
        if string.startswith(opt):
            return True


def convert_str(s):
    # with open(script_name) as f:
    markdown_cell = ''
    code_cell = ''
    nb = nbformat.v4.new_notebook()
    for line in f:
        if str_starts_with(line, ACCEPTED_CHARS):
            code_cell = new_cell(nb, code_cell)
            if str_starts_with(line, MARKDOWN_CHARS):
                # find the first occurence of |
                # and add the rest of the line to the markdown cell
                markdown_cell += line[line.index('|') + 1:]
            else:
                markdown_cell = new_cell(nb, markdown_cell, markdown=True)
        else:
            markdown_cell = new_cell(nb, markdown_cell, markdown=True)
            code_cell += line

    markdown_cell = new_cell(nb, markdown_cell, markdown=True)
    code_cell = new_cell(nb, code_cell)
    # return nbformat
    return nb
    # notebook_name = os.path.splitext(script_name)[0] + '.ipynb'
    # nbformat.write(nb, notebook_name)
    pass

def convert(script_name):
    """ Convert the python script to jupyter notebook"""
    with open(script_name) as f:
        markdown_cell = ''
        code_cell = ''
        nb = nbformat.v4.new_notebook()
        for line in f:
            if str_starts_with(line, ACCEPTED_CHARS):
                code_cell = new_cell(nb, code_cell)
                if str_starts_with(line, MARKDOWN_CHARS):
                    # find the first occurence of |
                    # and add the rest of the line to the markdown cell
                    markdown_cell += line[line.index('|') + 1:]
                else:
                    markdown_cell = new_cell(nb, markdown_cell, markdown=True)
            else:
                markdown_cell = new_cell(nb, markdown_cell, markdown=True)
                code_cell += line

        markdown_cell = new_cell(nb, markdown_cell, markdown=True)
        code_cell = new_cell(nb, code_cell)

        notebook_name = os.path.splitext(script_name)[0] + '.ipynb'
        nbformat.write(nb, notebook_name)


def parse_args():
    """Argument parsing for py2nb"""
    description = "Convert a python script to a jupyter notebook"
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument(
        "script_name",
        help="name of script (.py) to convert to jupyter notebook (.ipynb)")
    return parser.parse_args()


def main():
    args = parse_args()
    convert(args.script_name)


if __name__ == '__main__':
    main()


import os.path

from exam_generator.exam import Question, MCQuestion
from types import SimpleNamespace
import numpy as np

def uroll(bank):
    options = {'true': [], 'false': []}
    # ft = []
    # tt = []
    for k in bank:
        if k in ['true', 'false']:
            options[k] += bank[k]
        else:
            t, f = uroll(bank[k])
            options['true'] += t
            options['false'] += f
    return options['true'], options['false']


# class ManualMultipleChoice(MCQuestion):
#     def __init__(self, seed, bank, title='', template_base='explicit_question'):
#         super().__init__(seed, title=title)
#         self.template_base = template_base
#         self.bank = bank
#
#     def render(self, exam):
#         dirs = exam.get_dirs()
#         if not os.path.isfile(dirs.base + "/" + self.solution_file):
#             with open(dirs.base + "/" + self.solution_file, 'w') as f:
#                 f.write("Put your solution in " + self.solution_file)
#
#         return super().render(exam)
#
#     def generate(self):
#         x = SimpleNamespace()
#         # from numpy.random
#         itrue, ifalse = uroll(self.bank)
#
#         f = list(np.random.choice(ifalse, size=3, replace=False))
#         t = np.random.choice(itrue, size=1, replace=False)[0]
#         x.answers = [t] + f
#         x.static_solution = "statcic/"+self.solution_file
#
#         return x
#
#     pass

class RandomMultipleChoice(MCQuestion):

    def __init__(self, seed, bank, title='', solution_file='sol1.tex'):
        super().__init__(seed, title=title)
        self.solution_file = solution_file
        self.bank = bank

    def render(self, exam):
        dirs = exam.get_dirs()
        if not os.path.isfile(dirs.base + "/" + self.solution_file):
            with open(dirs.base + "/" + self.solution_file, 'w') as f:
                f.write("Put your solution in " + self.solution_file)

        return super().render(exam)

    def generate(self):
        x = SimpleNamespace()
        # from numpy.random
        itrue, ifalse = uroll(self.bank)

        f = list(np.random.choice(ifalse, size=3, replace=False))
        t = np.random.choice(itrue, size=1, replace=False)[0]
        x.answers = [t] + f
        x.static_solution = self.solution_file

        return x




from jinjafy import jinja_env
import numpy as np

def permute_exam_answers(section,permutation):
    permutation = list(permutation)
    if min(permutation) == 1:
        raise Exception(f"the specified permutation is invalid as it does not start with 0. {permutation}")
        # permutation = [i - 1 for i in permutation]
    # permutation.append(len(permutation))

    v = (" x" + section).split("\\item")
    v = v[1:]
    v = [s for s in v if len(s.strip()) > 0]

    if len(v) != len(permutation):
        print(section)
        print("num qs found:", len(v), permutation)
        assert False

    ls = [('\\choice' if p > 0 else '\\CorrectChoice')  + " " + v[p] for k, p in enumerate(permutation)]
    return ' '.join(ls)

def latex_env(env):
    import math
    env.globals['exp'] = math.exp
    env.globals['sqrt'] = math.sqrt
    env.globals['cos'] = math.cos
    env.globals['sin'] = math.sin

    env.globals['mround'] = mround
    env.globals['bold'] = bold
    env.globals['fmat'] = fmat
    env.globals['enumerate'] = enumerate
    env.globals['zip'] = zip
    env.globals['ensure_numpy'] = ensure_numpy
    env.globals['transpose'] = transpose
    import math
    env.globals['ceil'] = math.ceil
    env.globals['floor'] = math.floor


    from pylatexenc import latexencode
    env.globals['utf8tolatex'] = latexencode.utf8tolatex
    env.globals['as_set'] = jinja_env.as_set
    env.globals['as_set_list'] = jinja_env.as_set_list
    env.globals['len'] = jinja_env.mylen
    env.globals['get'] = jinja_env.jget
    env.globals['tolist'] = jinja_env.tolist
    filters = {}

    filters['as_set'] =  as_set
    filters['format_list'] =jinja_env.format_list
    filters['format_join'] = jinja_env.format_join
    filters['format_join_enum'] = jinja_env.format_join_enum
    filters['pm'] = lambda x: f" {x}" if x < 0 else f"+{x}"
    filters['bold'] = bold
    filters['capfirst'] = lambda x: (x[0].upper() + x[1:] if len(x) > 1 else x.upper()) if x != None and isinstance(x, str) else x
    filters['lowerfirst'] = lambda x: (x[0].lower() + x[1:] if len(x) > 1 else x.lower()) if x != None and isinstance(x, str) else x
    filters['infty'] = jinja_env.infty
    filters['n2w'] = jinja_env.n2w
    def latex_url(url):
        if not isinstance(url, str):
            return url
        url = url.replace("%", r"\%")
        return url
    filters['latex_url'] = latex_url
    filters['format_list_symbols'] = jinja_env.format_list_symbols
    filters['mround'] = mround
    def eround(val,l):
        x = str(mround(val, l))
        if l == 0:
            return x
        if '.' not in x:
            x = x + "."
        n = l - (len(x) - x.find(".") - 1)
        if n > 0:
            x = x + "0"*n
        return x

    filters['eround'] = eround
    filters['get'] = jinja_env.jget
    filters['flatten'] = jinja_env.flatten
    filters['aan'] = jinja_env.aan
    filters['bracket'] = bracket
    filters['tolist'] = jinja_env.tolist
    filters['rational'] = jinja_env.as_rational
    filters['permute_exam_answers'] = permute_exam_answers # Use my own for more freedom.
    env.filters.update(filters)

    ### 02465 stuff
    import sympy as sym
    from sympy import Dummy
    def flint(eq):
        """convert floats that are ints to ints"""
        reps = {}
        e = eq.replace(lambda x: x.is_Float and x == int(x), lambda x: reps.setdefault(x, Dummy()))
        return e.xreplace({v: int(k) for k, v in reps.items()})

    env.globals['latex'] = lambda eq: sym.latex(flint(eq))
    return env





def as_set(l):
    if type(l) != list and type(l) != np.ndarray:
        l = [l]
    l = list(l)
    s = [f'{i}' for i in l]
    s = '\{' + ", ".join(s) + "\}"
    return s


def bold(bob,d=True) :
    if not isinstance(bob, str) :
        bob = str(bob)
    if d :
        bob = '\\textbf{' + bob +"}"
    return bob


def fmat(bob,l=2,dobold=False) :
    bob = mround(bob,l)
    bob = bold(bob, dobold)
    return bob

def bracket(s):
    return "{"+str(s)+"}"

def un2str(x, xe, precision=2):
    """pretty print nominal value and uncertainty

        x  - nominal value
        xe - uncertainty
        precision - number of significant digits in uncertainty

        returns shortest string representation of `x +- xe` either as
        x.xx(ee)e+xx
        or as
        xxx.xx(ee)"""
    # base 10 exponents
    x_exp = int(floor(log10(x)))
    xe_exp = int(floor(log10(xe)))

    # uncertainty
    un_exp = xe_exp - precision + 1
    un_int = round(xe * 10 ** (-un_exp))

    # nominal value
    no_exp = un_exp
    no_int = round(x * 10 ** (-no_exp))

    # format - nom(unc)exp
    fieldw = x_exp - no_exp
    fmt = '%%.%df' % fieldw
    result1 = (fmt + '(%.0f)e%d') % (no_int * 10 ** (-fieldw), un_int, x_exp)

    # format - nom(unc)
    fieldw = max(0, -no_exp)
    fmt = '%%.%df' % fieldw
    result2 = (fmt + '(%.0f)') % (no_int * 10 ** no_exp, un_int * 10 ** max(0, un_exp))

    # return shortest representation
    if len(result2) <= len(result1):
        return result2
    else:
        return result1


def mround(val, l=2):
    if not isinstance(l, int):
        return un2str(val, l, 1)
    else:
        if isinstance(val, np.ndarray):
            return np.round(val * 10 ** l) / (10 ** l)
        else:
            return round(val * 10 ** l) / (10 ** l)


def transpose(X):
    return np.transpose( ensure_numpy( X) )


def ensure_numpy(X):
    if type(X) != np.ndarray:
        X = np.asarray(X)
    if X.ndim == 1:
        X = np.transpose( np.expand_dims(X,1) )
    return X
